import importlib
from copy import deepcopy
import yoyo


async def connect_models():
    python_models = yoyo.tools.listdir(__file__)

    for el in deepcopy(python_models):
        if not el[-3:] == ".py":
            python_models.remove(el)
    python_models.remove("__init__.py")
    python_models = list(map(lambda x: x[:-3], python_models))

    package = yoyo.tools.package(__file__)

    for module in python_models:
        import_connect = importlib.import_module(f".{module}", package=package).main
        await import_connect()



import sys
import os
import logging
from datetime import datetime


class Logger:
    EXCEPTION = 100
    CRITICAL = 50
    ERROR = 40
    WARNING = 30
    INFO = 20
    DEBUG = 10
    NOTSET = 0

    def __init__(self):
        today = datetime.strftime(datetime.now(), "%d-%m-%y-%H_%M_%S")

        os.makedirs("logs", exist_ok=True)
        log_path = f"logs/log-{today}.log"

        # 🔹 Set default logging config (your requested format)
        logging.basicConfig(
            level=logging.INFO,
            format=u'%(filename)s:%(funcName)s:%(lineno)d '
                   '#%(levelname)-8s [%(asctime)s] - %(name)s - %(message)s',
            datefmt="%d/%b/%y %H:%M:%S",
        )

        logger_name = "YoYoLogger"
        self.__log = logging.getLogger(logger_name)

        # Avoid re-adding handlers
        if not self.__log.handlers:
            # 🔹 Console handler
            console_handler = logging.StreamHandler(sys.stderr)
            console_handler.setLevel(logging.INFO)
            console_handler.setFormatter(logging.Formatter(
                u'%(filename)s:%(funcName)s:%(lineno)d '
                '#%(levelname)-8s [%(asctime)s] - %(name)s - %(message)s',
                datefmt="%d/%b/%y %H:%M:%S"
            ))

            # 🔹 File handler
            file_handler = logging.FileHandler(log_path, mode="w", encoding="utf-8")
            file_handler.setLevel(logging.DEBUG)
            file_handler.setFormatter(logging.Formatter(
                u'%(filename)s:%(funcName)s:%(lineno)d '
                '#%(levelname)-8s [%(asctime)s] - %(name)s - %(message)s',
                datefmt="%d/%b/%y %H:%M:%S"
            ))

            # Attach both
            self.__log.addHandler(console_handler)
            self.__log.addHandler(file_handler)

        self.__log.setLevel(self.INFO)

        # Map levels to functions
        self.__methods_map = {
            self.DEBUG: self.__log.debug,
            self.INFO: self.__log.info,
            self.WARNING: self.__log.warning,
            self.ERROR: self.__log.error,
            self.CRITICAL: self.__log.critical,
            self.EXCEPTION: self.__log.exception,
        }

    def __call__(self, lvl, msg, *args, **kwargs):
        """Log with the given level."""
        if lvl in self.__methods_map:
            self.__methods_map[lvl](msg, *args, **kwargs)
        else:
            self.__log.log(lvl, msg, *args, **kwargs)

    def set_level(self, level=None):
        if level is None:
            level = self.INFO
        self.__log.setLevel(level)



import asyncio

from aiogram import Bot, Dispatcher
from aiogram.fsm.storage.base import DefaultKeyBuilder
from aiogram.fsm.storage.memory import MemoryStorage
from aiogram.fsm.storage.redis import RedisStorage
from aiogram.webhook.aiohttp_server import setup_application
from aiohttp import web
from redis import Redis

from tgbot.bots.{bot_name}.config import get_config


def build() -> tuple[Bot, Dispatcher]:
    config = get_config()

    bot = Bot(
        token=config.tgbot.token
    )

    if config.tgbot.use_redis:
        redis = Redis(
            host=config.redis.host,
            port=config.redis.port,
            password=config.redis.password
        )
        key_builder = None
        if config.redis.is_prefix:
            key_builder = DefaultKeyBuilder(prefix=config.redis.prefix)
        storage = RedisStorage(redis, key_builder=key_builder)
    else:
        storage = MemoryStorage()

    dp = Dispatcher(
        storage=storage
    )

    from tgbot.bots.{bot_name}.scripts import register_scripts
    register_scripts(bot, dp)
    from tgbot.bots.{bot_name}.utils.scheduler import register_scheduler
    register_scheduler(bot)
    from tgbot.bots.{bot_name}.routers import register_routers
    register_routers(dp)

    return bot, dp


async def start(bot: Bot, dp: Dispatcher):
    from tgbot.bots.{bot_name}.models import connect_models
    await connect_models()

    from tgbot.bots.{bot_name}.utils.menu import register_my_commands
    await register_my_commands(bot)

    config = get_config()

    if config.tgbot.use_webhook:

        async def on_startup(app):
            await bot.set_webhook(app["webhook_url"])

        async def on_cleanup(app):
            await bot.delete_webhook()
            await dp.storage.close()
            await bot.session.close()

        app = web.Application()
        app["bot"] = bot
        app["dp"] = dp
        app["webhook_url"] = config.webhook.url

        setup_application(app, dp, bot=bot)
        app.on_startup.append(on_startup)
        app.on_cleanup.append(on_cleanup)

        runner = web.AppRunner(app)
        await runner.setup()
        site = web.TCPSite(runner, "0.0.0.0", 8080)
        await site.start()

        try:
            await asyncio.Event().wait()
        finally:
            await runner.cleanup()

    else:
        try:
            await dp.start_polling(bot)
        finally:
            await dp.storage.close()
            await bot.session.close()


__all__ = ['build', 'start']



from dataclasses import dataclass
from environs import Env
from yoyo.logger import Logger


@dataclass
class Tgbot:
    token: str
    admins: list[int]
    use_redis: bool
    use_webhook: bool


@dataclass
class Redis:
    host: str
    port: int
    password: str
    is_prefix: bool
    prefix: str


@dataclass
class Webhook:
    url: str


@dataclass
class Payment:
    token: str
    currency: str


@dataclass
class Misc:
    tz: str


@dataclass
class Config:
    tgbot: Tgbot
    redis: Redis
    payment: Payment
    misc: Misc


def get_config() -> Config:
    env = Env()
    path = './tgbot/bots/{bot_name}/.env'
    env.read_env(path)

    return Config(
        tgbot=Tgbot(
            token=env.str("TOKEN"),
            admins=list(map(int, env.str("ADMINS").split(' '))),
            use_redis=env.bool("USE_REDIS"),
            use_webhook=env.bool("USE_WEBHOOK"),
        ),
        redis=Redis(
            host=env.str("REDIS_HOST"),
            port=env.int("REDIS_PORT"),
            password=env.str("REDIS_PASSWORD"),
            is_prefix=env.bool("REDIS_IS_PREFIX"),
            prefix=env.str("REDIS_PREFIX"),
        ),
        payment=Payment(
            token=env.str("PAYMENT_PROVIDER_TOKEN"),
            currency=env.str("PAYMENT_CURRENCY"),
        ),
        misc=Misc(
            tz=env.str("SCHEDULER_TZ")
        )
    )


log = Logger()




"""
All Language Codes
IETF Language Tags

en - English
ru - Russian
uk - Ukrainian
be - Belarusian
ca - Catalan
hr - Croatian
nl - Dutch
fr - French
de - German
id - Indonesian
it - Italian
kk - Kazakh
ko - Korean
ms - Malay
pl - Polish
pt - Portuguese (Brazil)
sr - Serbian
es - Spanish
tr - Turkish
"""

DEFAULT_LANGUAGE = 'en'  # Ensure that all texts include the default language compatibility

FIRST_TEXT = {
	'en': "Hello!",
	'ru': "Привет!"
}  # Example of a text

def get_text(lang_code: str, text: dict[str, str]) -> str:
    if not lang_code in text.keys():
        lang_code = DEFAULT_LANGUAGE
    return text[lang_code]



import importlib
import yoyo

from aiogram import Router


def register_handlers(router: Router):
    modules: list[str] = list(map(lambda x: x[:-3], yoyo.tools.listdir(__file__)))
    modules.remove('__init__')
    try:
        modules.remove('__pycach')
    except ValueError:
        pass

    package = yoyo.tools.package(__file__)
    for module in modules:
        import_register = importlib.import_module(f".{module}", package=package).register
        import_register(router)



import importlib
from src import yoyo

from aiogram import Dispatcher, Router


def register_routers(dp: Dispatcher):
    # Register middlewares of dispatcher (global middlewares)

    # Include routers to dispatcher
    routers_list = yoyo.tools.listdir(__file__)
    routers_list.remove("__init__.py")
    try:
        routers_list.remove("__pycache__")
    except ValueError:
        pass

    routers: list[tuple[int, Router]] = []


    package = yoyo.tools.package(__file__)
    for module in routers_list:
        import_get_router = importlib.import_module(f".{module}", package=package).get_router
        router.append(import_get_router())

        max_i = max(i for i, _ in routers if i != -1) + 1
        for _, router in sorted(routers, key=lambda x: max_i if x[0] == -1 else x[0]):
            dp.include_router(router)



import aiosqlite as asql
from aiosqlite import Connection


class {capital_name}:
    _db: Connection
    name = r"{db_path}"

    @classmethod
    async def connect(cls):
        cls._db = await asql.connect(cls.name)
        c = await cls._db.cursor()
        await c.execute("""CREATE TABLE IF NOT EXISTS {name}Table ();""")
        await cls._db.commit()


async def main():
    await {capital_name}.connect()




import argparse

from rich.console import Console

from _yoyo_cli.core import init_project, help_command, create_bot, establish_router, create_model


def main():
    # Initializing custom Console by rich module
    console = Console()

    # Creating parser and subparsers for CLI
    parser = argparse.ArgumentParser(prog="yoyo", description="🛠 YoYoGram CLI — Generate Templates")
    subparsers = parser.add_subparsers(dest="command")

    # Help Command
    subparsers.add_parser("help", help="Show list of available commands")

    # Init Command
    subparsers.add_parser("init", help="Create a new project")

    # Bot Command
    bot_parser = subparsers.add_parser("bot", help="Create a new bot")
    bot_parser.add_argument("name", help="The name of the bot")

    # Router Command
    router_parser = subparsers.add_parser("router", help="Establish a new router")
    router_parser.add_argument("name", help="The name of the router")
    router_parser.add_argument("index", nargs='?', help="The index of the router", default=-1)
    router_parser.add_argument("bot_name", nargs='?', help="The name of the bot", default=None)

    # Model Command
    model_parser = subparsers.add_parser("model", help="Create a new model")
    model_parser.add_argument("name", help="The name of the model")
    model_parser.add_argument("bot_name", nargs='?', help="The name of the bot to which model will be created", default=None)

    # Get Request
    args = parser.parse_args()

    # Handle Request
    if args.command == "init":
        init_project(console)
    elif args.command == "bot":
        create_bot(console, args.name)
    elif args.command == "router":
        establish_router(console, args.name, args.index, args.bot_name)
    elif args.command == "model":
        create_model(console, args.name, args.bot_name)
    elif args.command == "help" or args.command is None:
        help_command(console)
    else:
        help_command(console)


import importlib
import yoyo

from aiogram import Bot, Dispatcher


async def build_bots() -> list[tuple[Bot, Dispatcher]]:
    bots_list = yoyo.tools.listdir(__file__)
    bots_list.remove("__init__.py")
    try:
        bots_list.remove("__pycache__")
    except:
        pass

    bots: list[tuple[Bot, Dispatcher]] = []

    package = yoyo.tools.package(__file__)

    for module in bots_list:
        import_build_bot = importlib.import_module(f".{module}", package=package).build_bot
        bots.append(import_build_bot())

    return bots


async def start_bots(bots: list[tuple[Bot, Dispatcher]]):
    bots_list = yoyo.tools.listdir(__file__)


    bots_list.remove("__init__.py")
    try:
        bots_list.remove("__pycache__")
    except:
        pass

    package = yoyo.tools.package(__file__)

    for module, bot_dp in zip(bots_list, bots):
        bot, dp = bot_dp
        import_start_bot = importlib.import_module(f".{module}", package=package).start
        await import_start_bot(bot, dp)



from aiogram import Bot, Dispatcher

from tgbot.bots import build_bots, start_bots


async def build() -> list[tuple[Bot, Dispatcher]]:
    from tgbot.models import connect_models
    await connect_models()

    return await build_bots()


async def start(bots: list[tuple[Bot, Dispatcher]]):
    await start_bots(bots)


async def main():
    await start(await build())



import shutil
from pathlib import Path
import importlib.resources as pkg_resources

from rich.console import Console
from rich.table import Table

from _yoyo_cli import templates


def get_project_root() -> Path:
    current = Path.cwd()
    for parent in [current] + list(current.parents):
        if (parent / "main.py").exists() or (parent / ".git").exists():
            return parent
    return current


def remove_keep_files(root: Path) -> None:
    for path in root.rglob("*"):
        if path.is_file() and path.name == ".keep":
            if any(part in ("venv", ".venv", "__pycache__") for part in path.parts):
                continue
            path.unlink()


def init_project(console: Console) -> None:
    dst_path = get_project_root()
    with pkg_resources.path(templates, "project") as template_path:
        shutil.copytree(template_path, dst_path, dirs_exist_ok=True)
        (dst_path / "__init__.py").unlink(missing_ok=True)

    remove_keep_files(dst_path)

    console.print("[green]✅ Project has been created![/green]")


def help_command(console: Console) -> None:
    table = Table(title="🛠 Доступные команды YoYoGram CLI", style="magenta")
    table.add_column("Command", style="cyan3", no_wrap=True)
    table.add_column("Arguments", style="deep_sky_blue1")
    table.add_column("Description", style="white")
    table.add_row("init", "", "Create a new project")
    table.add_row("bot", "bot_name", "Create a new bot")
    table.add_row("router", "name Optional:index Optional:bot_name\nProviding bot_name it will create router by tgbot/bots/bot_name/routers/router_name path\nNot providing bot_name, it established router by the current path\nNot providing index means the last position", "Establish a new router")
    table.add_row("model", "name Optional:bot_name\nWithout bot_name it will create it in tgbot dir, making access for all bots to use the model", "Create a new model")
    console.print(table)


def create_bot(console: Console, bot_name: str) -> None:
    with pkg_resources.path(templates, "bot") as template_path:
        bots_dir = get_project_root() / "tgbot" / "bots" / bot_name

        for item in template_path.rglob("*"):
            if item.is_file():
                rel_path = item.relative_to(template_path)
                dst_file = bots_dir / rel_path
                dst_file.parent.mkdir(parents=True, exist_ok=True)

                try:
                    content = item.read_text(encoding="utf-8")
                    content = content.replace("{bot_name}", bot_name)
                    dst_file.write_text(content, encoding="utf-8")
                except UnicodeDecodeError:
                    shutil.copyfile(item, dst_file)

        remove_keep_files(bots_dir)
        (bots_dir / "utils" / "__init__.py").unlink(missing_ok=True)

        console.print(f"[green]✅ {bot_name} has been created by tgbot/bots/{bot_name} directory![/green]")


def establish_router(console: Console, router_name: str, router_index: int = -1, bot_name: str = None) -> None:
    with pkg_resources.path(templates, "router") as template_path:
        if bot_name:
            if (get_project_root() / "tgbot" / "bots" / bot_name).exists():
                router_dir = get_project_root() / "tgbot" / "bots" / bot_name / "routers" / router_name
            else:
                console.print(f"[red]❌ There is no {bot_name} bot![/red]")
        else:
            bots = [p for p in (get_project_root() / "tgbot" / "bots").iterdir() if p.is_dir()]
            if len(bots) == 1:
                router_dir = bots[0] / "routers" / router_name
            else:
                for parent in [Path.cwd()] + list(Path.cwd().parents):
                    if (parent / "routers").exists():
                        router_dir = parent / "routers" / router_name
                        bot_name = parent.name
                        break
                else:
                    console.print("[red]❌ You're not in certain bot, so please provide the bot_name parameter![/red]")
                    return

        for item in template_path.rglob("*"):
            if item.is_file():
                rel_path = item.relative_to(template_path)
                dst_file = router_dir / rel_path
                dst_file.parent.mkdir(parents=True, exist_ok=True)
                try:
                    content = item.read_text(encoding="utf-8")
                    content = content.replace("{bot_name}", bot_name)
                    content = content.replace("{router_index}", str(router_index))
                    dst_file.write_text(content, encoding="utf-8")
                except UnicodeDecodeError:
                    shutil.copyfile(item, dst_file)

        console.print(f"[green]✅ {router_name} has been established by tgbot/bots/{bot_name}/routers/{router_name} directory![/green]")


def create_model(console: Console, name: str, bot_name: str = None) -> None:
    if bot_name:
        model_path = get_project_root() / "tgbot" / "bots" / bot_name / "models" / f"{name}.py"
        db_path = f"tgbot/bots/{bot_name}/models/{name}.db"
    else:
        model_path = get_project_root() / "tgbot" / "models" / f"{name}.py"
        db_path = f"tgbot/models/{name}.db"

    name = name.lower()
    capital_name = name[0].upper() + name[1:]

    with pkg_resources.path(templates, "model") as template_path:
        for item in template_path.rglob("model.py"):
            model_path.parent.mkdir(parents=True, exist_ok=True)

            content = item.read_text(encoding="utf-8")
            content = content.replace("{name}", name)
            content = content.replace("{capital_name}", capital_name)
            content = content.replace("{db_path}", db_path)
            model_path.write_text(content, encoding="utf-8")

    if bot_name:
        console.print(f"[green]✅ {capital_name} Model has been created by tgbot/bots/{bot_name}/models directory![/green]")
    else:
        console.print(f"[green]✅ {capital_name} Model has been created by tgbot/models directory![/green]")



import os
import sys
from pathlib import Path


def listdir(file: __file__):
	return os.listdir(os.path.dirname(file))

def listpath(file: __file__):
	path = Path(file).resolve()
	parts = list(path.parts)
	if parts[0].endswith("\\"):
		parts[0] = parts[0][:-1]
	return parts

def package(file: __file__):
	project_root = Path(sys.modules['__main__'].__file__).resolve().parent
	current_path = Path(file).resolve()
	relative_path = current_path.relative_to(project_root).parent
	return ".".join(relative_path.with_suffix('').parts)

def basename(file: __file__):
	return file.split(os.sep)[-2]
